<?php
/**
 * Module Name: Featured Gallery Slider
 * Description: Provides transition effects between images
 * Version: 0.9a
 * Author: digitalnature
 * Author URI: http://digitalnature.eu
 */


// class name must follow this pattern (AtomMod + directory name)
class AtomModFeaturedGallery extends AtomMod{

  private $effects;

  // available public variables from parent class:
  //
  // $this->url  - this module's url path
  // $this->dir  - this module's directory

  public function init(){

    $this->effects = array(
       'Fade'             => _a('Fade'),
       'SlideH'           => _a('Slide Horizontally'),
       'SlideV'           => _a('Slide Vertically'),
       'RandomTiles'      => _a('Random Tiles'),
       'HorizontalTiles'  => _a('Horizontal Tiles'),
       'DiagonalTiles'    => _a('Diagonal Tiles'),
       'Spiral'           => _a('Spiral'),
       'RandomStripes'    => _a('Random Stripes'),
       'TileWave'         => _a('Tile Wave'),
       'StripeWave'       => _a('Stripe Wave'),
       'Curtain'          => _a('Curtain'),
       'Interweave'       => _a('Interweave'),
    );

    // register extra theme options
    Atom::app()->addDefaultOptions(array(
      'featured_image_size'       => '960x320',
      'featured_source'           => 'marked',
      'featured_link'             => 'post',
      'featured_effects'          => array_keys($this->effects),
      'featured_count'            => 6,
      'featured_delay'            => 15,
      'featured_timeframe'        => 60,
      'featured_caption'          => true,
      'featured_caption_content'  => 'post_filtered',
      'featured_navigation'       => true,
      'featured_pager'            => true,
      'featured_visibility'       => array(
         'page-home'          => 1,
         'user-visitor'       => 1,
         'role-administrator' => 1,
         'role-editor'        => 1,
         'role-author'        => 1,
         'role-contributor'   => 1,
         'role-subscriber'    => 1,
      ),
    ));

    if(is_admin())
      // insert a tab, 35 is the priority (somewhere between content options and ads)
      Atom::admin()->addTab('featured', _a('Featured gallery'), array(&$this, 'form'), 35);

    // front-end hooks
    Atom::add('before_main',          array(&$this, 'output'));
    add_action('wp_loaded',           array(&$this, 'imageSizes'));
  }

  // gallery image size
  public function imageSizes(){
    list($w, $h) = explode('x', Atom::app()->options('featured_image_size'));
    add_image_size('featured_gallery', $w, $h, true);
  }

  // tab entry in theme settings
  public function form($settings){
    $wp_page_types = array(
      'home'     => _a('Blog Homepage'),
      'search'   => _a('Search Results'),
      'author'   => _a('Author Archives'),
      'date'     => _a('Date-based Archives'),
      'tag'      => _a('Tag Archives'),
      'category' => _a('Category Archives'),
    );

    $settings = Atom::app()->options();

    ?>
    <!-- tab: featured -->
    <div class="clear-block">

      <div class="notice e">
        This module is still in alpha stage, some features might not be implemented yet.
      </div>

      <div class="notice">
        <strong><?php _ae("Important:"); ?></strong> <?php printf(_a("Images must have at least a %s pixel resolution."), $settings['featured_image_size']); ?>
      </div>

      <div class="clear-block">

       <div class="block alignleft">
         <h3 class="title"><?php _ae("Content source"); ?></h3>

         <div class="entry">
         <label for="featured_source_marked">
           <input type="radio" value="marked" id="featured_source_marked" name="featured_source" <?php checked($settings['featured_source'], 'marked'); ?> />
           <?php printf(_a("%s that I mark as featured"), '<a href="'.admin_url('edit.php').'">'._a("Media items").'</a>'); ?>
         </label>
         </div>
         <div class="entry">
         <label for="featured_source_nextgen" class="disabled" title="Not yet implemented">
           <input disabled="disabled" id="featured_source_nextgen" name="featured_source" type="radio" value="nextgen" <?php checked($settings['featured_source'], 'nextgen'); ?> />
           <?php _ae("NextGEN gallery"); ?>
           <select disabled="disabled">
             <option><?php _ae("-- All galleries --"); ?></option>
           </select>
         </label>
         </div>

         <h3 class="title"><?php _ae("Options"); ?></h3>
         <div class="entry">
          <label for="featured_count"><?php _ae('Number of items to show:') ?></label>
          <input id="featured_count" name="featured_count" type="text" value="<?php echo (int)$settings['featured_count']; ?>" size="3" />
         </div>

         <div class="entry">
          <?php $input = '<input id="featured_timeframe" name="featured_timeframe" type="text" value="'.(int)$settings['featured_timeframe'].'" size="3" />'; ?>
          <label for="featured_timeframe"><?php printf(_a('Exclude items older than %s day(s)'), $input); ?></label>
         </div>

         <div class="entry">
          <?php $input = '<input id="featured_delay" name="featured_delay" type="text" value="'.(int)$settings['featured_delay'].'" size="3" />'; ?>
          <label for="featured_delay"><?php printf(_a('Auto-Cycle every %s second(s)'), $input); ?></label>
          <br />
          <small><?php _ae("(0 to disable auto-play)"); ?></small>
         </div>

         <div class="entry">
          <label for="featured_link"><?php _ae('Slides link to:') ?></label>
          <select id="featured_link" name="featured_link">
           <option value="image" <?php selected($settings['featured_link'], 'image'); ?>><?php _ae("Original image"); ?></option>
           <option value="post" <?php selected($settings['featured_link'], 'post'); ?>><?php _ae("Attached post, if any"); ?></option>
           <option value="" <?php selected($settings['featured_link'], ''); ?>><?php _ae("Nothing"); ?></option>
          </select>
         </div>

         <h3 class="title"><?php _ae("Show"); ?></h3>
         <div class="entry">
          <input type="hidden" name="featured_caption" value="0" />
          <input id="featured_caption" name="featured_caption" type="checkbox" followRules <?php checked($settings['featured_caption']); ?> value="1" />
          <label for="featured_caption"><?php _ae('Captions:') ?></label>
          <select id="featured_caption_content" followRules rules="DEPENDS ON featured_caption" name="featured_caption_content">
           <option value="post_filtered" <?php selected($settings['featured_caption_content'], 'post_filtered'); ?>><?php _ae("Filtered content from the attached post, if any"); ?></option>
           <option value="post_excerpt" <?php selected($settings['featured_caption_content'], 'post_excerpt'); ?>><?php _ae("Excerpt from the attached post, if any"); ?></option>
           <option value="image" <?php selected($settings['featured_caption_content'], 'image'); ?>><?php _ae("Image title and description"); ?></option>
          </select>
         </div>

         <div class="entry">
          <label for="featured_navigation">
          <input type="hidden" name="featured_navigation" value="0" />
          <input type="checkbox" id="featured_navigation" name="featured_navigation" <?php checked($settings['featured_navigation']); ?> value="1" />
   	    <?php _ae('Previous/Next links (on mouse over)'); ?></label>
         </div>

         <div class="entry">
          <label for="featured_pager">
          <input type="hidden" name="featured_pager" value="0" />
          <input type="checkbox" id="featured_pager" name="featured_pager" <?php checked($settings['featured_pager']); ?> value="1" />
   	    <?php _ae('Pager (on mouse over)'); ?></label>
         </div>

        <h3 class="title"><?php _ae("Effects"); ?></h3>
        <input type="hidden" name="featured_effects" value="0" />
        <?php foreach($this->effects as $key => $label): ?>
         <div class="entry">
           <label for="featured_effects_<?php echo $key; ?>">
             <input id="featured_effects_<?php echo $key; ?>" name="featured_effects[]" type="checkbox" value="<?php echo $key; ?>" <?php checked(true, in_array($key, (array)$settings['featured_effects'])); ?> />
             <?php echo $label; ?>
           </label>
         </div>
        <?php endforeach; ?>

       </div>

       <div class="block alignright">

        <h3 class="title"><?php _ae("Visible on:"); ?></h3>

        <?php
         foreach($wp_page_types as $key => $label): ?>
         <div class="entry">
          <input type="hidden" name="featured_visibility[page-<?php echo $key; ?>]" value="0" />
          <input type="checkbox" value="1" <?php checked(!empty($settings["featured_visibility"]["page-{$key}"])); ?> id="featured_visibility[page-<?php echo $key; ?>]" name="featured_visibility[page-<?php echo $key; ?>]" />
          <label for="featured_visibility[page-<?php echo $key; ?>]"><?php echo $label; ?></label>
         </div>

        <?php endforeach; ?>

        <?php foreach(get_post_types(array('public' => true)) as $post_type):
          $object = get_post_type_object($post_type);
          if(empty($object->labels->name) || $post_type == 'page') continue; // we handle pages separately ?>
         <div class="entry">
          <input type="hidden" name="featured_visibility[page-singular-<?php echo $post_type; ?>]" value="0" />
          <input type="checkbox" value="1" <?php checked(!empty($settings["featured_visibility"]["page-singular-{$post_type}"])); ?> id="featured_visibility[page-singular-<?php echo $post_type; ?>]" name="featured_visibility[page-singular-<?php echo $post_type; ?>]" />
          <label for="featured_visibility[page-singular-<?php echo $post_type; ?>]"><?php printf(_a('Single: %s'), $object->labels->name); ?></label>
         </div>
        <?php endforeach; ?>

        <?php foreach(get_taxonomies(array('public' => true, '_builtin' => false)) as $taxonomy):
          $object = get_taxonomy($taxonomy);
          if(empty($object->labels->name)) continue; ?>
         <div class="entry">
          <input type="hidden" name="featured_visibility[page-tax-<?php echo $taxonomy; ?>]" value="0" />
          <input type="checkbox" value="1" <?php checked(!empty($settings["featured_visibility"]["page-tax-{$taxonomy}"])); ?> id="featured_visibility[page-tax-<?php echo $taxonomy; ?>]" name="featured_visibility[page-tax-<?php echo $taxonomy; ?>]" />
          <label for="featured_visibility[page-tax-<?php echo $taxonomy; ?>]"><?php printf(_a('Tax Archive: %s'), $object->labels->name); ?></label>
         </div>
        <?php endforeach; ?>

        <?php
          $pages = get_pages();
          if($pages):
            echo '<div class="entry">';
            foreach($pages as $page): ?>
             <div class="entry">
              <input type="hidden" name="featured_visibility[page-<?php echo $page->ID; ?>]" value="0">
              <input type="checkbox" <?php checked(!empty($settings["featured_visibility"]["page-{$page->ID}"])) ?> id="featured_visibility[page-<?php echo $page->ID; ?>]" name="featured_visibility[page-<?php echo $page->ID; ?>]" value="1" />
              <label for="featured_visibility[page-<?php echo $page->ID; ?>]">
              <?php printf(_a("Page: %s"), '<a href="'.get_permalink($page).'" target="_blank"><strong title="'.$page->post_title.'">'.((strlen($page->post_title) > 16) ? substr($page->post_title, 0, 16)."..." : $page->post_title).'</strong></a>'); ?>
              </label>
             </div>
            <?php endforeach;
            echo '</div>';
          endif;
        ?>

        <h3 class="title"><?php _ae("To:"); ?></h3>

      <div class="entry">
        <input type="hidden" name="featured_visibility[user-visitor]" value="0" />
        <input type="checkbox" value="1" <?php checked(!empty($settings["featured_visibility"]['user-visitor'])); ?> id="featured_visibility[user-visitor]" name="featured_visibility[user-visitor]" />
        <label for="featured_visibility[user-visitor]"><?php _ae("Unregistered user (Visitor)"); ?></label>
      </div>

      <?php
       $wp_roles = new WP_Roles();
       foreach($wp_roles->role_names as $role => $label):
        ?>
       <div class="entry">
         <input type="hidden" name="featured_visibility[role-<?php echo $role; ?>]" value="0" />
         <input type="checkbox" value="1" <?php checked(!empty($settings["featured_visibility"]["role-{$role}"])); ?> id="featured_visibility[role-<?php echo $role; ?>]" name="featured_visibility[role-<?php echo $role; ?>]" />
         <label for="featured_visibility[role-<?php echo $role; ?>]"><?php echo $label; ?></label>
       </div>
       <?php endforeach; ?>

       </div>
      </div>

    </div>
    <!-- /tab: featured -->
    <?php
  }


  public function output(){

    $settings = Atom::app()->options();
    if(!atom_strict_visibility_check($settings['featured_visibility'])) return;

    $output = '';
    $media_ids = wp_parse_id_list(get_option('featured_media'));

    shuffle($media_ids);
    $count = 1;

    $effects = (is_array($settings['featured_effects']) && !empty($settings['featured_effects'])) ? $settings['featured_effects'] : array('fade');
    shuffle($effects);

    $caption_push = array('bottom', 'left', 'right');
    $caption_push_counter = 0;
    shuffle($caption_push);

    foreach($media_ids as $media_id){

      $image = wp_get_attachment_image_src($media_id, 'featured_gallery');
      if(!$image) continue;

      list($src, $width, $height) = $image;

      list($featured_width, $featured_height) = explode('x', $settings['featured_image_size']);
      if(($width < $featured_width) || ($height < $featured_height)) continue;

      $html = '';
      $media = &get_post($media_id);
      $post_parent = $media->post_parent;

      if($post_parent)
        Atom::app()->setCurrentPost($post_parent);

      $alt = trim(strip_tags(get_post_meta($media_id, '_wp_attachment_image_alt', true)));
      if(empty($alt) && $post_parent) $alt = get_the_excerpt();
      if(empty($alt) && $post_parent) $alt = Atom::app()->post->getTitle();

      $attr = array(
        'src'   => $src,
        'class' => 'photo',
        'alt'   => $alt,
      );

      $attr['title'] = $post_parent ? Atom::app()->post->getTitle() : $media->post_title;
      $attr = apply_filters('wp_get_attachment_image_attributes', $attr, $media_id);
      $attr = array_map('esc_attr', $attr);
      $html = rtrim('<img '.image_hwstring($width, $height));
      foreach($attr as $name => $value) $html .= " {$name}=\"{$value}\"";
      $html .= ' />';

//      $caption = empty($post_parent->post_content) ? $post_parent->post_excerpt : $post_parent->post_content;

      //if($link == 'post' && ($parent = get_post_field('post_parent', $media_id))) $link = get_permalink($parent);
      //elseif($link == 'image' && is_array($src = wp_get_attachment_image_src($media_id, 'full'))) $link = $src[0];
      //else $link = '';

      //if($link) $html = '<a href="'.$link.'" title="'.strip_tags($caption).'">'.$html.'</a>';

      //$html = '<a href="#">'.$html.'</a>';

      if($settings['featured_caption']){
        $desc = $title = '';
        switch($settings['featured_caption_content']){
          case 'post_filtered':
            if($post_parent) $title = '<h3>'.Atom::app()->post->getTitle().'</h3>';
            if($post_parent) $desc = Atom::app()->post->getContent(60, array(
              'allowed_tags' => array('a', 'abbr', 'acronym', 'b', 'cite', 'code', 'del', 'dfn', 'em', 'i', 'ins', 'q', 'strong', 'sub', 'sup'),
              'more' => '[&hellip;]',
            ));
          break;

          case 'post_excerpt':
            if(!empty($attached_post->post_excerpt)) $desc = apply_filters('the_excerpt', $attached_post->post_excerpt);
          break;

          case 'image':

          break;
        }

        if($desc || $title) $html .= '<div class="caption push-'.$caption_push[$caption_push_counter++].'"><div class="content">'.$title.$desc.'</div></div>';
        if($caption_push_counter > 2) $caption_push_counter = 0;
      }

      // used up all effects? reset then...
      if(empty($effect_history)) $effect_history = $effects;

      // select a random effect and remove it from the history of effects so we don't choose it again in the next loop
      $effect = array_pop($effect_history);

      $output .= '<div class="slide '.(($count === 1) ? 'first' : '').'" data-fx="'.$effect.'" data-delay="'.(int)$settings['featured_delay'].'">'.$html.'</div>';

      // stop if jquery is disabled (one slide only), or if the current slide count exceeds the number limit from the options
      if(!Atom::app()->options('jquery') || ($count === $settings['featured_count'])) break;

      $count++;
    }

    // reset post data
    Atom::app()->resetCurrentPost();

    if(empty($output)) return Atom::app()->addDebugMessage("Featured content: No relevant entries found");
    ?>
    <!-- featured gallery -->

    <div id="featured" class="<?php if($count > 1): ?>iSlider count-<?php echo $count - 1; ?><?php else: ?>single<?php endif; ?>">
      <?php echo $output; ?>
    </div>

    <!-- /featured gallery -->

    <?php
  }

}
